--
-- counter with overflow
-- Jan Pospisil
--   2010; KP, FEL, CVUT
--   2015; CTP, ALICE, CERN
--   2016; BE-BI-QP, CERN
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity CounterO is
	generic (
		g_Width: natural := 32
	);
	port (
		Clk_ik: in std_logic;
		Reset_ir: in std_logic;
			-- synchronous reset
		Enable_i: in std_logic;
			-- counts
		Overflow_o: out std_logic := '0';
			-- '1' for 1 T_CLK when overflowed
		Value_ob: out unsigned
	);
end entity;

architecture syn of CounterO is

	signal Value_b: unsigned(g_Width-1 downto 0) := (others => '0');
	
begin

	pCounter: process (Clk_ik) is
		variable NewValue: unsigned(g_Width-1 downto 0) := (others => '0');
	begin
		if rising_edge(Clk_ik) then
			if Reset_ir = '1' then
				NewValue := (others => '0');
				Overflow_o <= '0';
			else
				Overflow_o <= '0';
				if Enable_i = '1' then
					NewValue := Value_b + 1;
					if to_integer(NewValue) = 0 then
						Overflow_o <= '1';
					end if;
				end if;
			end if;
			Value_b <= NewValue;
		end if;
	end process pCounter;

	Value_ob <= Value_b;

end architecture;