--
-- minimal pulse generator
-- (generated pulse can be longer)
--

library ieee;
use ieee.std_logic_1164.all;

entity pulseGenerator is
	generic (
		PULSE_MIN_WIDTH: integer := 3 -- multiply of 'clk' periods
	);
	port (
		clk: in std_logic;
		reset: in std_logic;
		signalIn: in std_logic;
		pulseOut: out std_logic
	);
end entity;

architecture base of pulseGenerator is

	signal sReg: std_logic_vector(PULSE_MIN_WIDTH-1 downto 0) := (others => '0');

begin

	pShiftRegister: process (clk) begin
		if rising_edge(clk) then
			if reset = '1' then
				sReg <= (others => '0');
			else
				sReg <= sReg(PULSE_MIN_WIDTH-2 downto 0)&signalIn;
			end if;
		end if;
	end process;
	
	pPulseGenerator: process (sReg) is
		variable pulse: std_logic := '0';
	begin
		pulse := '0';
		for i in sReg'range loop
			pulse := pulse or sReg(i);
		end loop;
		pulseOut <= pulse;
	end process;

end architecture;
