library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity pulseGeneratorTime is
    generic (
        CLK_FREQ: real;             -- input clock frequency in Hz
        PULSE_MIN_WIDTH_TIME: time  -- minimal pulse width in [s]
    );
    port (
        clk: in std_logic;
        reset: in std_logic;
        signalIn: in std_logic;
        pulseOut: out std_logic
    );
end entity;

architecture syn of pulseGeneratorTime is

    constant CLK_PERIOD: time := (1.0 sec)/CLK_FREQ;

    --! computes least possible width of the vector to store value X
    -- http://stackoverflow.com/a/12751341/615627
    function f_log2(x: positive) return natural is
        variable i: natural;
    begin
        i := 0;
        while (2**i < x) loop
            i := i + 1;
        end loop;
        return i;
    end function;

    function calculatePulseClk (CLK_PERIOD, PULSE_MIN_WIDTH_TIME: time) return integer is
        variable result: integer;
    begin
        result := PULSE_MIN_WIDTH_TIME / CLK_PERIOD;
        if result < 1 then
            result := 1;
        end if;
        return result;
    end function;

    constant PULSE_MIN_WIDTH: integer := calculatePulseClk(CLK_PERIOD, PULSE_MIN_WIDTH_TIME);
    
begin

    cPulseGenerator: entity work.pulseGenerator(base)
        generic map (
            PULSE_MIN_WIDTH => PULSE_MIN_WIDTH
        )
        port map (
            clk => clk,
            reset => reset,
            signalIn => signalIn,
            pulseOut => pulseOut
        );

end architecture;
