library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.FfpgPkg.all;

entity FfpgCoreWrapper is
    generic (
        g_ClkFrequency: natural -- input clock frequency in MHz
    );
    port (
        -- Wishbone connection
        clk_sys_i: in std_logic;
        rst_i: in std_logic;
        wb_adr_i: in std_logic_vector(31 downto 0);
        wb_dat_i: in std_logic_vector(31 downto 0);
        wb_dat_o: out std_logic_vector(31 downto 0);
        wb_cyc_i: in std_logic;
        wb_sel_i: in std_logic_vector(3 downto 0);
        wb_stb_i: in std_logic;
        wb_we_i: in std_logic;
        wb_ack_o: out std_logic;
        wb_stall_o: out std_logic;
        TriggerDacFrameSynchronization_n: out std_logic;
        TriggerDacSerialClock: out std_logic;
        TriggerDacSerialData: out std_logic;
        VcxoDacFrameSynchronization_n: out std_logic;
        VcxoDacSerialClock: out std_logic;
        VcxoDacSerialData: out std_logic;
        ch1_out_en: out std_logic;
        ch2_out_en: out std_logic;
        d: out unsigned(9 downto 0);
        ch_1_le_set: out std_logic;
        ch_1_le_res: out std_logic;
        ch_2_le_set: out std_logic;
        ch_2_le_res: out std_logic;
        Ch1Set_o: out std_logic;
        Ch1Res_o: out std_logic;
        Ch2Set_o: out std_logic;
        Ch2Res_o: out std_logic;
        Trigger_i: in std_logic
    );
end entity;

architecture syn of FfpgCoreWrapper is

    signal TriggerDac: t_Ad5600Interface;
    signal VcxoDac: t_Ad5600Interface;
    signal WbMosi: t_WbInterfaceMosi;
    signal WbMiso: t_WbInterfaceMiso;
    
begin

    WbMosi.Clk <= clk_sys_i;
    WbMosi.Rst <= rst_i;
    WbMosi.Adr_b32 <= wb_adr_i;
    WbMosi.Dat_b32 <= wb_dat_i;
    WbMosi.Cyc <= wb_cyc_i;
    WbMosi.Sel_b4 <= wb_sel_i;
    WbMosi.Stb <= wb_stb_i;
    WbMosi.We <= wb_we_i;

    cFfpgCore: entity work.FfpgCore(syn)
        generic map (
            g_ClkFrequency => g_ClkFrequency
        )
        port map (
            Wb_i => WbMosi,
            Wb_o => WbMiso,
            TriggerDac_o => TriggerDac,
            VcxoDac_o => VcxoDac,
            ch1_out_en => ch1_out_en,
            ch2_out_en => ch2_out_en,
            d => d,
            ch_1_le_set => ch_1_le_set,
            ch_1_le_res => ch_1_le_set,
            ch_2_le_set => ch_2_le_set,
            ch_2_le_res => ch_2_le_res,
            Ch1Set_o => Ch1Set_o,
            Ch1Res_o => Ch1Res_o,
            Ch2Set_o => Ch2Set_o,
            Ch2Res_o => Ch2Res_o,
            Trigger_i => Trigger_i
        );

    wb_dat_o <= WbMiso.Dat_b32;
    wb_ack_o <= WbMiso.Ack;
    wb_stall_o <= WbMiso.Stall;

    TriggerDacFrameSynchronization_n <= TriggerDac.FrameSynchronization_n;
    TriggerDacSerialClock <= TriggerDac.SerialClock;
    TriggerDacSerialData <= TriggerDac.SerialData;

    VcxoDacFrameSynchronization_n <= VcxoDac.FrameSynchronization_n;
    VcxoDacSerialClock <= VcxoDac.SerialClock;
    VcxoDacSerialData <= VcxoDac.SerialData;

end architecture;
